import { SQRT_OF_TWO, isZeroVector } from "./util.js"
import Actor from "./actor.js"

export default class Player extends Actor {
  constructor(game, x, y) {
    super(game)
    this.x = x
    this.y = y
    this.width = 32
    this.height = 32
    this.xVel = 0
    this.yVel = 0
    this.color = "#56E"
    this.playerDirection = { x: 0, y: 1 }
    this.interactHitbox = null
  }

  tick(dt) {
    this.handleInputAndMovement()
  }

  handleInputAndMovement() {
    const dir = this.inputDirection()
    this.xVel = dir.x * 2
    this.yVel = dir.y * 2

    this.x += this.xVel 
    this.y += this.yVel 

    if (this.collidesWithAbsolutelyAnything()) {
      this.x -= this.xVel
      this.y -= this.yVel
    }

    if (!isZeroVector(dir)) this.playerDirection = dir

    if (this.isInputPressed("interact")) this.createInteractHitbox()
    else this.interactHitbox = null
  }

  collidesWithAbsolutelyAnything() {
    return this.collidesWithTiles() || this.collidesWithObjects()
  }

  collidesWithObjects() {
    const objects = this.game.currentRoom.objectsUnderRectangle(this)
    return !!objects.find(object => object.collides())
  }

  collidesWithTiles() {
    const tur = this.game.currentRoom.tilesUnderRectangle(this).filter(x => x)
    const colliders = tur.filter(tile => tile.properties.find(prop => prop.name == "collides" && prop.value))
    return !!colliders.length
  }

  createInteractHitbox() {
    this.interactHitbox = {
      width: this.width,
      height: this.height,
      x: this.x + (this.playerDirection.x * this.width / 2),
      y: this.y + (this.playerDirection.y * this.height / 2),
    }
  }

  isInputPressed(action) {
    return this.game.input.isInputPressed.call(this.game.input, action)
  }

  inputDirection() {
    const dir = { x: 0, y: 0 }
    if (this.isInputPressed("up")) dir.y -= 1
    if (this.isInputPressed("down")) dir.y += 1
    if (this.isInputPressed("left")) dir.x -= 1
    if (this.isInputPressed("right")) dir.x += 1

    if (Math.abs(dir.x, dir.y) == 2) {
      dir.x *= SQRT_OF_TWO
      dir.y *= SQRT_OF_TWO
    }

    return dir
  }

  draw(ctx) {
    this.color = `rgb(128 ${(this.playerDirection.x * 128) + 128} ${(this.playerDirection.y * 128) + 128}`
    ctx.fillStyle = this.color
    ctx.fillRect(this.x, this.y, this.width, this.height)
    if (this.interactHitbox) {
      ctx.fillStyle = "#FF000088"
      ctx.fillRect(this.interactHitbox.x, this.interactHitbox.y, this.interactHitbox.width, this.interactHitbox.height)
    }
  }
}
