export default class Tileset {
  constructor(game, json, name, source) {
    this.game = game
    this.json = json
    this.name = name
    this.source = source
    this.tiles = json.tiles || []
  }

  get imagesToLoad() {
    const image = {}
    image[`${this.name}-image`] = this.json.image
    return image
  }

  populateImage(assets) {
    this.image = assets.get(`${this.name}-image`)
  }

  get tileWidth() {
    return this.json.tilewidth
  }

  get tileHeight() {
    return this.json.tileheight
  }

  get columns() {
    return this.json.columns
  }

  tileOffset(index) {
    return [
      ((index % this.columns) * (this.tileWidth + this.json.spacing)),
      (Math.floor(index / this.columns) * (this.tileHeight + this.json.spacing))
    ]
  }

  tileAt(index) {
    return this.tiles.find(tile => tile.id == index)
  }

  collides(index) {
    const tile = this.tileAt(index)
    if (!tile) return

    return tile.properties.find(prop => prop.name == "collides").value == "true"
  }
}
